var $fileTreeContainer;
var $hdf5TreeContainer;
var $bokehVisContainer;
var pathFromTree;
var hdf5PathFromTree;

var reportContent;

function countReport() {
    var counter = 0;
    for (var i in reportContent) {
        counter += reportContent[i].length;
    }
    return counter;
}

function copyToClipboard(element) {
    var $temp = $("<textarea>");
    $("body").append($temp);
    $temp.val($(element).text()).select();
    document.execCommand("copy");
    $temp.remove();
}


$(document).ready(function(){
    $fileTreeContainer = $('#jstree-dir');  // select file tree container
    $hdf5TreeContainer = $('#jstree-hdf5'); // select hdf5 tree container
    $bokehVisContainer = $('#visualize-hdf5'); // select bokeh visualization container
    pathFromTree = "";
    $("#importCode code").html("Please select a dataset.");

    $("#importCode").click(function() {
      copyToClipboard("#importCode");
      $(this).tooltip("dispose");
      //$(this).tooltip({data-placement:"left"});
      $(this).tooltip({title:"copied!"});
      $(this).tooltip("show");
    } );
    $('#importCode').mouseenter( function(){
      $(this).tooltip("dispose");
      $(this).tooltip({title:"click to copy", placement:"left"});
      $(this).tooltip("show");
    } ).mouseleave( function(){
      $(this).tooltip("hide");
    } );



    reportContent = {};

    // hide report buttons
    $("#addToRepButton").hide();
    $("#printReportBtn").hide();
    $("#importInNotebook").hide();
    $("#notebook").hide();

    $("#importCode").hide();


    $("#showNotebook").click(function(){

       $("#importCode").toggle();
       $("#notebook").toggle();
    });

    $('#importInNotebook').click(function() {

      $('#importCode').toggle()
    })


    $("#printReportBtn").click(function(){
      var $modalContent = $('#reportModal').find("#reportContent");
      $modalContent.html("");
      for (var i in reportContent) {
          var file = reportContent[i];
          for (var f in file) {
            var reportData = file[f];
            $modalContent.append("<h4>Filename: "+reportData.filename+"</h4>");
            $modalContent.append(reportData.canvas);
            var $ul = $("<ul>");
            $ul.append("<li><strong>Dataset:</strong> "+reportData.dataset+"</li>");
            $modalContent.append($ul);
          }
      }
      $('#reportModal').modal('show');
    });

    $bokehVisContainer.parent().on( "click", "#addToRepButton", function() {
      var elementData = $(this).data();
      if(typeof(reportContent[elementData.filename])=="undefined") {
        reportContent[elementData.filename] = [];
      }

      reportContent[elementData.filename].push({
        'dataset': elementData.dataset,
        'filename': elementData.filename,
        'url': elementData.url,
        'mime': elementData.mime,
        'z': elementData.z,
      });

      if($("#visualize-hdf5 canvas").length) {
          html2canvas(document.querySelector("#visualize-hdf5")).then(function(canvas) {
              reportContent[elementData.filename][reportContent[elementData.filename].length-1].canvas = canvas;
              $("#reportCounter").html(countReport());

              $("#printReportBtn").show();
              $("#importInNotebook").show();
              $("#importCode").show();
              $("#notebook").show();
          });
      } else {
          reportContent[elementData.filename][reportContent[elementData.filename].length-1].canvas = null;
          $("#reportCounter").html(countReport());
          $("#printReportBtn").show();
          $("#importInNotebook").show();
          $("#importCode").show();
          $("#notebook").show();
      }


    });

    $("#showDirTreeButton").hide();
    $("#showDirTreeButton").click(function(){
      $(this).hide();
      $fileTreeContainer.show();
    });

    var jstreeIcons = {
        "types" : {
          "default" : {
            "icon" : "fa fa-folder"
          },
          "file" : {
            "icon" : "fa fa-file"
          }
        },
        "plugins" : [ "types" ]
    };
    jsondirtree = $.extend( jsondirtree, jstreeIcons );

    //collapsing
    $("#toggleFileTree").click( function(){
        $fileTreeContainer.parent().toggle();
    });

    $("#toggleHdf5Tree").click( function(){
        $hdf5TreeContainer.parent().toggle();
    });

    // file browser
    $fileTreeContainer
    .on('changed.jstree', function (e, data) { updateHdf5Tree(data); }) // call function updateHdf5Tree when tree is clicked
    .jstree( jsondirtree ); // init file tree on the selected html container

    $hdf5TreeContainer
    .on('changed.jstree', function (e, data) { clickHdf5Tree(data); }) // call function clickHdf5Tree when tree is clicked
    .jstree( ); // init hdf5 tree on the selected html container

    $(window).resize(function(){
         generateBokehPlot(1);
     });


});

function initFileTree() {
}

function clickHdf5Tree(data) {
    if ( (Object.keys(data).length > 0) && (typeof(data.node)!="undefined") ) {

        //get the clicked node text value and log
        hdf5PathFromTree = data.node.text;

        var importCodeScript = "!pip install six==1.13.0 numpy==1.19.1 h5py==2.10.0\n"+
                        "import h5py\n" +
                        "import numpy as np"
                        "filename, dataset = './data/"+pathFromTree+"', '"+hdf5PathFromTree+"'\n"+
                        "with h5py.File(filename, 'r') as hf:\n"+
                        "    data = np.array(hf.get(dataset))\n "

        $("#importCode code").html(importCodeScript);

        //hide file tree
        $fileTreeContainer.hide();
        $("#showDirTreeButton").show();
        generateBokehPlot(0);
    } else {
      console.log("no data received!");
      console.log(data);
    }
}

function generateBokehPlot(z) {
    // prepare URL to be called for visualization
    // template: /visualize/<FILENAME>,dataset=<DATASETPATH>
    var hdfCall = "/visualize/?filename="+pathFromTree+"&dataset="+hdf5PathFromTree+"&z="+z;

    $(".overlay").removeClass("hidden");
    $("#addToRepButton").hide();
    $("#importInNotebook").hide();
    $("#importCode").hide();
    $("#notebook").hide()

    $.get( hdfCall, function( bokeh ) {
        $bokehVisContainer.html(bokeh);
        $(".overlay").addClass("hidden");
        if(bokeh != "Not a hdf5 dataset.") {
            $("#addToRepButton").data("dataset",hdf5PathFromTree);
            $("#addToRepButton").data("filename",pathFromTree);
            $("#addToRepButton").data("z",z);
            $("#addToRepButton").data("url","");
            $("#addToRepButton").data("mime","");
            $("#addToRepButton").show();
            $("#importInNotebook").show();
            $("#notebook").show();
        }
    });
}

function updateHdf5Tree(data)  {

    // clear content of bokeh visualization
    $bokehVisContainer.html("");
    //get path from clicked node
    pathFromTree = data.instance.get_path(data.node,'/');
    //and log that
    //create url with path for openhdf5 app route
    var url = "/openhdf5/"+pathFromTree;
    //log that
    $("#addToRepButton").hide();
    $("#importInNotebook").hide();
    $("#importCode").hide();
    $("#notebook").hide();

    // hdf5 browser
    $.get( url, function( hdf5tree ) {
        console.log("received reply from url with data: ");
        console.log(hdf5tree);
        // received hdf5 tree from python
        // retrieve hdf5 structure, convert to json, log
        hdf5tree = JSON.parse(hdf5tree);
        // update content of hdf5tree
        $hdf5TreeContainer.jstree(true).settings = hdf5tree;
        $hdf5TreeContainer.jstree(true).refresh();

        if(!hdf5tree.core.data.length) {
            // no  hdf5 tree received from python
            // maybe it's a file?
            // let's ask flask for file info
            $.get( "/file_info/"+pathFromTree, function( file_info_response ) {
                // file_info_response = JSON.parse(file_info_response);
                if(file_info_response.mime_type.indexOf("image")>-1) {
                    $bokehVisContainer.html('<img class="img-responsive" style="height: 75vh; max-width: 100%;" src="'+file_info_response.url+'" >');
                } else if (file_info_response.mime_type.indexOf("video")>-1) {
                    $bokehVisContainer.html('<video class="img-responsive" style="height: 75vh; max-width: 100%;" video-element" controls><source type="'+file_info_response.mime_type+'" src="'+file_info_response.url+'"></video>');
                } else if (file_info_response.mime_type.indexOf("text")>-1) {
                    $bokehVisContainer.html('<iframe src="'+file_info_response.url+'" style="height: 75vh; max-width: 100%;">');
                } else {
                    $bokehVisContainer.html("I don't know how to preview "+pathFromTree);
                }
                $("#addToRepButton").data("dataset","");
                $("#addToRepButton").data("filename",pathFromTree);
                $("#addToRepButton").data("z",0);
                $("#addToRepButton").data("url",file_info_response.url);
                $("#addToRepButton").data("mime",file_info_response.mime_type);
                $("#addToRepButton").show();
            });

        }

    });


    /**/
}
